--***********************************************************************************
--* Date Struct v1.3
--* by Ofer Zelichover (c) 6/2002
--* based on algorithms found at:
--* http://alcor.concordia.ca/~gpkatch/gdate-algorithm.html
--*
--* Laset Update: Januar 21, 2012
--******************************************************************************
--*  You may use this script freely as you see fit.
--*  You may use parts or the script as a whole in your own scripts.
--*  (it would be nice if you give me a credit if you do so ;))
--******************************************************************************
--*  This script comes with no waranty!
--*  Although I tried this script and couldn't find any problems with it, I can
--*  in no way be held responsible for any kind of loss or damage, whether
--*  direct or indirect, due to the use of this script.
--*
--*  ********************************************************************
--*  *** IF YOU DON'T LIKE THE ABOVE STATEMENT, DON'T USE THIS SCRIPT ***
--*  ********************************************************************
--*
--*  **** This script was written for max 4.2 and wasn't tested on other ****
--*  **** versions of max.                                               ****
--*
--*  If you find any bugs in this script, please let me know.
--******************************************************************************
--*
--* Description:
--* ------------
--* The date_struct is a set of functions for dealing with dates. It allows convertion
--* between date<->integer and different display formats.
--*
--*
--* Variables and function descriptions:
--* -------------------------------------
--* Most of these functions, as well as the struct itself use a format variable. This format
--* var is a string describing the date format (from here on I'll reffere to the date format var as DF.)
--* This come to solve the problem of different systems using different display formats for
--* dates. The DF string should have the date field charachters (y for year, m for month and d for day)
--* in any order, and separator charachters seperating the field chars. The separators can be
--* any charachter (or set of characters) except 'y', 'm' and 'd'.
--* Some of the functions return and/or can recieve date array. A date array is an array in the
--* format: #(year, month, day).
--*
--*
--*
--* struct date_struct
--* var:
--*		<string>format - the struct DF string.
--*
--*
--* fn <string/array>getSeparators [<DF string>fromFormat:] [<name>returnAs:#string]
--* description: returns the separators from the DF string. if returnAs is #string
--* 		the funtion will return a string with the separators for use with the filterString function.
--* 		If returnAs is #array, the function will return an array with the separators.
--* 		NOTE: this functions is mainly for the struct internal use.
--* vars:
--* 	fromFormat: - an optional DF string. if not supplied, the struct's internal DF is used.
--* 	returnAs:#string - an optional input to determine the type of var the function will return.
--* 		possible values are:
--* 		#string - return a string of the separators (default).
--* 		#array - returns an array of the separators.
--*
--*
--* fn toDateArray <string>date [<DF string>fromFormat:]
--* description: converts a DF string into a date array.
--* vars:
--* 	date - a string containing the date.
--* 	fromFormat: - an optional DF string. if not supplied, the struct's internal DF is used.
--*
--*
--* fn <integer>dateToInt <string/array>date [<DF string>fromFormat:]
--* description: converts a date into an integer.
--* vars:
--* 	date - a string containing the date or a date array.
--* 	fromFormat: - an optional DF string. if not supplied, the struct's internal DF is used.
--*
--*
--* fn <date array>intToDate <integer>i
--* description: converts integer into a date array.
--* vars:
--* 	i - the date as an integer.
--*
--*
--* fn <boolean>isDateLegal <string/array>date [<DF string>fromFormat:]
--* description: returns true if date is a leagal date, false otherwise.
--* vars:
--* 	date - a string containing the date or a date array.
--* 	fromFormat: - an optional DF string. if not supplied, the struct's internal DF is used.
--*
--*
--* fn <string>toFormat <string/array>date [<DF string>fromFormat:] [<DF string>toFormat:]
--* description: converts a date (either a string date or date array) from one format to another.
--* vars:
--* 	date - a string containing the date or a date array.
--* 	fromFormat: - an optional DF string describing the input DF. if not supplied, the struct's internal DF is used.
--* 	toFormat: - an optional DF string describing the output DF. if not supplied, the struct's internal DF is used.
--*
--*
--* fn <string>toMDYFormat <string/array>date
--* description: converts a date (either a string date or date array) from the struct's
--* 		internal DF to m/d/y format.
--* vars:
--* 	date - a string containing the date or a date array.
--*
--*
--*	fn getSystemDateFormat =
--*	description: Get the system date format from the registry
--*
--*	
--*	fn getSystemDateSeparator
--* description: Get the system date separator from the registry
--*
--*
--* fn <date array>getSystemDate [<DF string>systemFormat:]
--* description: returns the system date as a date array.
--* vars:
--* 	date - a string containing the date or a date array.
--* 	systemFormat: - an optional DF string describing the system DF. if not supplied, the struct's internal DF is used.
--*
--************************************************************************************/


--**********************************************************************************
-- Start of struct definition
--**********************************************************************************
struct s_Date (
	-- variables
	------------
	format="m/d/y", -- the date format (m-month, d-day, y-year)

	-- functions
	-------------
	-- get the format separator charachters.
	fn getSeparators fromFormat: returnAs:#string = (
		local frmt = if fromFormat==unsupplied then format else fromFormat
		local sep
		case returnAs of (
			#string: (
				sep = ""
				for c = 1 to frmt.count where (frmt[c]!="m" AND frmt[c]!="M" AND frmt[c]!="d" AND frmt[c]!="D" AND frmt[c]!="y" AND frmt[c]!="Y") do
					sep+=frmt[c]
			)
			#array: (
				sep = #()
				local str = ""
				for c = 1 to frmt.count do (
					if (frmt[c]=="m" OR frmt[c]=="M" OR frmt[c]=="d" OR frmt[c]=="D" OR frmt[c]=="y" OR frmt[c]=="Y") then (
						if c>1 then (
							if str!="" then append sep str
							str = ""
						)
					) else (
						str += frmt[c]
					)
				)
				if str != "" then append sep str
			)
		)
		sep
	),

	-- converts from the struct fromat to an array in #(y,m,d) format.
	fn toDateArray date fromFormat: = (
		date = date as string
		if fromFormat==unsupplied then fromFormat=format
		local dateArr = filterString date (getSeparators fromFormat:fromFormat)
		if dateArr.count != 3 then return undefined
		local formatArr = filterString fromFormat (getSeparators fromFormat:fromFormat)
		local y=0, m=0, d=0
		for i = 1 to formatArr.count do (
			if (findString formatArr[i] "y")!=undefined OR (findString formatArr[i] "Y")!=undefined then y=i
			if (findString formatArr[i] "m")!=undefined OR (findString formatArr[i] "M")!=undefined then m=i
			if (findString formatArr[i] "d")!=undefined OR (findString formatArr[i] "D")!=undefined then d=i
		)
		if m==0 OR d==0 OR y==0 then return undefined
		#(dateArr[y] as integer, dateArr[m] as integer, dateArr[d] as integer)
	),

	fn dateToInt date fromFormat: = (
		if fromFormat==unsupplied then fromFormat=format
		if classOf date!=array then dd = toDateArray date fromFormat:fromFormat
		else dd = date
		local y=dd[1], m=dd[2], d=dd[3]
		m = (mod (m + 9) 12) as integer
		y -= m/10
		(365*y + y/4 - y/100 + y/400 + (m*306 + 5)/10 + d - 1)
	),

	fn intToDate i = (
		i = i as integer
		local y = ((i + 1.4780)/365.2425) as integer
		local ddd = i - (365*y + y/4 - y/100 + y/400)
		if ddd < 0 then (
			y -= 1
			ddd = i - (365*y + y/4 - y/100 + y/400)
		)
		local mi = (100*ddd + 52)/3060
		local mm = (mod (mi + 2) 12) as integer + 1
		y += (mi + 2)/12
		local dd = ddd - (mi*306 + 5)/10 + 1
		#(y as integer , mm as integer, dd as integer)
	),

	fn isDateLegal date fromFormat: = (
		if fromFormat==unsupplied then fromFormat=format
		if classOf date!=array then dd = toDateArray date fromFormat:fromFormat
		else dd = date
		local y=dd[1], m=dd[2], d=dd[3]
		local dd = intToDate((dateToInt #(y, m, d)))
		y==dd[1] AND m==dd[2] AND d==dd[3]
	),

	-- converts from m/d/y format or from an array format ( #(year,month,day) )to the struct format.
	fn toFormat date fromFormat: toFormat: = (
		if toFormat==unsupplied then toFormat=format
		if fromFormat==unsupplied then fromFormat=format
		local dd = if classOf date!=array then (toDateArray date fromFormat:fromFormat) else date
		if dd==undefined or not isDateLegal dd then return undefined
		local sep = getSeparators fromFormat:toFormat returnAs:#array
		if sep==#() or sep==undefined then sep=#("/", "/")
		local formatArr = filterString toFormat (getSeparators fromFormat:toFormat returnAs:#string)
		if formatArr.count != 3 then return undefined
		local outStr = ""
		for i = 1 to 3 do (
			if formatArr[i].count > 1 then (
				local repeatingChar = true
				local firstChar = formatArr[i][1]
				for j = 1 to formatArr[i].count do
					repeatingChar = repeatingChar and findString formatArr[i][1] formatArr[i][j] != undefined
				if repeatingChar then
					formatArr[i] = firstChar
			)
			case formatArr[i] of (
				"y": outStr += dd[1] as string
				"Y": outStr += dd[1] as string
				"m": outStr += dd[2] as string
				"M": outStr += dd[2] as string
				"d": outStr += dd[3] as string
				"D": outStr += dd[3] as string
			)
			if i < 3 then outStr += sep[i]
		)
		outStr
	),

	-- converts from the struct fromat to m/d/y format.
	fn toMDYFormat date = (
		toFormat date toFormat:"m/d/y"
	),

	fn toYMDFormat date = (
		toFormat date toFormat:"y/m/d"
	),

	
	-- Get the system date format from the registry
	fn getSystemDateFormat =
	(
		-- First check if there is access to the registry
		if isKindOf registry StructDef then (
			local k, v
			registry.openKey HKEY_CURRENT_USER "Control Panel\\International" accessRights:#readOnly key:&k
			registry.queryValue k "sShortDate" value:&v
			registry.closeKey k
			-- Return the registry value
			return v
		) else (
			-- Retrun "m/d/y" as the default format when there's no access to the registry.
			return "m/d/y"
		)
	),

	-- Get the system date separator from the registry
	fn getSystemDateSeparator =
	(
		-- First check if there is access to the registry
		if isKindOf registry StructDef then (
			local k, v
			registry.openKey HKEY_CURRENT_USER "Control Panel\\International" accessRights:#readOnly key:&k
			registry.queryValue k "sDate" value:&v
			registry.closeKey k
			-- Return the registry value
			return v
		) else (
			-- Retrun "/" as the default separator when there's no access to the registry.
			return "/"
		)
	),
	
	
	-- Initialize the format with the system format
	fn __initFormat =
	(
		format = getSystemDateFormat()
		undefined
	),
	
	_initFormat = __initFormat(),
	

	fn getSystemDate systemFormat: = (
		if systemFormat==unsupplied then systemFormat=format
		toDateArray ((filterString localTime " ")[1]) fromFormat:systemFormat
	)
) -- end date_struct
--**********************************************************************************
-- End of struct definition
--**********************************************************************************
